# About

Mod Style is for use on Modernize's S3 microsites. It's purpose is to streamline development by providing flexible reusable style components such as:

* SASS color variables
* media breakpoints
* buttons and button groups
* input fields including: radio labels, checkboxes, dropdowns, text inputs etc.
* icons, fonts, and typography
* logos for both desktop and mobile devices
* and other form elements

It's functionality is built on top of the bootstrap-sass npm package, which is a sass-powered version of [Bootstrap 3](https://getbootstrap.com/docs/3.3/getting-started/).

An interactive user interface with a full range of examples can be found at [modify.modernize.com](https://modify.modernize.com). This page also has a sandbox environment for testing changes to the package, which can be found at [modifyqa.modernize.com](https://modifyqa.modernize.com).

This design system is based on the design system sketch file that is authored and maintained by our designers. The current version can be found here: [Design System Sketch File](https://drive.google.com/drive/folders/0B183MmrG_1B7MUx1YlNEVnJjSWc), alongside a changelog, a list of archived versions, and versions that are currently in progress. If this link is not accessible, or has not been shared with you, please let someone on the engineering team know.

## Structure

* assets/
  * icons/ - icon font folder
    * modicons-basic/
    * modicons-fill/
    * modicons-stroke/
  * logos/
  * styles/
    * base/ - minimum core css files used by every page
      * animations
      * breakpoints
      * colors
      * fonts
      * index - all base components imported to this file to make it easier to import them all at once
      * reset
    * components/
      * buttons
      * images
      * index - all base components imported to this file to make it easier to import them all at once
      * inputs
      * logos
      * nav
    * icons/
      * basics
      * fille
      * index
      * stroke
    * vendor/
      * bootstrap-base - for importing minimum core bootstrap modules for all pages
      * custom-variables - for overwriting bootstraps variables with our own styles
* .editorconfig - rules for code editors based on company's code guidelines
* .sass-lint.yml - configuration for SASS/SCSS linter
* package-lock.json - lock file for package.json; makes sure that the same versions of Node modules are installed across env
* package.json - Node modules used in mod-site

## Installation

### For projects using mod-site:

1. Ensure that the latest version of [mod-site](https://bitbucket.modernize.com/projects/MS/repos/mod-site/browse) (currently 4.0.*) is a dependency of your project inside you package.json.
  * mod-site ensures that:
    * The most recent version of [mod-style-bootstrap](https://www.npmjs.com/package/mod-style-bootstrap) is installed as a dependency in your project.
    * [bootstrap-sass](https://www.npmjs.com/package/bootstrap-sass) is also included as a dependency in your project.
    * Gulp handles proper compiling of assets and files from mod-style into your project.
2. Please add mod-style-assets to your `.gitignore` file.
3. Create a folder named `vendor` inside the `src/styles/` folder of your project.
4. Inside the `vendor` folder, create a file called `bootstrap-base.scss`.
5. Inside this file you will import the `bootstrap-base.scss` file from this package.
6. Note: If you need any additional bootstrap components (see below for full list of available components), you will import them from the bootstrap-sass package into this file.
7. Add this base file into the head section of your html file using a link tag.
8. Your custom css for your project should be listed as a link tag **AFTER** the bootstrap-base file.

**Example: properly including link tags in html file**
```html
<!DOCTYPE html>
<html lang="en">
	<head>
    <link href="/styles/vendor/bootstrap-base.css" rel="stylesheet" type="text/css" />
    <link href="/styles/home.css" rel="stylesheet" type="text/css" />
  </head>
  <body>
```

**Example: importing bootstrap base file from mod-style**
```css
@import '../../../node_modules/mod-style-bootstrap/assets/styles/vendor/bootstrap-base';
```

**Example: importing bootstrap components from bootstrap-sass**
```css
@import '../../../node_modules/bootstrap-sass/assets/stylesheets/bootstrap/code';
```


### For projects not using mod-site:

* Currently this functionality is not supported.

## Bootstrap Sass

For more information on the bootstrap-sass npm package, file structure, or other general implementation questions please see the [Bootstrap 3 - Using Sass Documentation](https://getbootstrap.com/docs/3.3/css/#sass)

For more about available components within Bootstrap 3 see [Bootstrap 3 Components](https://getbootstrap.com/docs/3.3/components/).

Another version of this list can be found inside the [bootstrap-sass repo](https://github.com/twbs/bootstrap-sass/tree/master/assets/stylesheets/bootstrap).


#### styles/vendor/bootstrap-base.scss

The mod-style [bootstrap-base.scss](https://bitbucket.modernize.com/projects/MS/repos/mod-style/browse/assets/styles/vendor/bootstrap-base.scss) file serves to pull in the minimum bootstrap components necessary for all pages.

* The base components included in this file are:
  * custom-variables - *Note: see custom variables section below for more info*
  * variables
  * mixins
  * normalize
  * scaffolding
  * component-animations
  * grid
  * forms
  * buttons
  * input-groups
  * modals - *Note: some javascript files necessary for this to work, see [bootstrap modals](https://getbootstrap.com/docs/3.3/javascript/#modals) for more info*
  * utilities
  * responsive-utilities

##### List of all other available bootstrap components

Reset and dependencies
* print
* glyphicons

Core css
* type
* code
* tables

Components
* dropdowns - *Note: We use our own custom css for these elements*
* button-groups
* navs
* navbar
* breadcrumbs
* pagination
* pager
* labels
* badges
* jumbotron
* thumbnails
* alerts
* progress-bars
* media
* list-group
* panels
* responsive-embed
* wells
* close

Components w/ JS
* tooltip
* popovers
* carousel

#### styles/vendor/custom-variables.scss

Every Sass variable in Bootstrap includes the `!default` flag allowing you to override the variable’s default value in your own Sass without modifying Bootstrap’s source code.

For that reason mod-style contains a copy of the [bootstrap-sass variable.scss file](https://github.com/twbs/bootstrap-sass/blob/master/assets/stylesheets/bootstrap/_variables.scss) located in `styles/vendor/custom-variables.scss`. In this file the `!default` flag has been removed from all variables so that it won’t be re-assigned by the default values in Bootstrap. All variables that are in use by our pages have been over written with our own custom styles.

This file is included at the top of the `bootstrap-base.scss` file because it must come before you import Bootstrap’s Sass variables and other files for the override to work properly.


## Icons

We use [Icomoon](https://icomoon.io/app/#/select) for importing and exporting our custom font library.

## Developing and testing locally

To test and develope locally you will need to work within two seperate repositories:

**modify.modernize.com**
1. Please ensure that you have cloned the [modify.modernize.com repo](https://bitbucket.modernize.com/projects/MS/repos/modify.modernize.com/browse) and you have pulled the most recent changes from master.
2. Please make sure to branch from master using a branch name that corresponds with a JIRA ticket number, for ease of tracking changes.
3. Symlink your local copy of mod-style with this project by running `npm link mod-style-bootstrap #` from your terminal, in the root directory of this project. *(Note: This link allows you to see changes made within mod-style when you refresh the page)*.
4. Run gulp serve from the root directory of this project.
5. Make necessary changes. If new features or elements are being added (i.e icons, form-elements etc.) please be sure to update this repo to reflect these changes.
6. Be sure to unlink mod-style before pushing your changes by running `npm unlink mod-style-bootstrap #` in the modify.modernize.com directory.
7. Create a PR against master branch
8. Once your PR is approved merge your mod-style feature branch into qa.
9. Once qa approves you can merge your changes to master.


**mod-style**
1. Please ensure that you have pulled the most recent changes from master.
2. Please make sure to branch from master using a branch name that corresponds with the same JIRA ticket number you used above, for ease of tracking changes.
3. Make necessary changes. If new features or elements are being added (i.e icons, form-elements etc.) please be sure to update the modify.modernize.com ui to reflect these changes.
4. Update the version number in the package.json.
5. If this update coincides with an update to the design system sketch file, please be sure to update `sketchFileVersion` in the package.json as well, to match the version number named on the sketch file.
6. Create a PR against master branch.
7. Once your PR is approved merge your mod-style feature branch directly to master *(Note: There is no qa branch for mod-style)*.
8. In your terminal, from the root directory of mod-style, on the master branch run `npm publish`.
9. Depending on the version number you are releasing, the change will take effect on pages only once the page has been re-deployed to production by qa. If it's a major version increase, pages will have to be manually opted in to the change.


## License

This project is licensed under the MIT license.

Copyright (c) 2019 Modernize.
